tolerance = 0.25;
ball_radius =  (24.25 + 8.16*2)/2;
holder_inset = 1;
holder_length = 5;
holder_height = 9;
holder_gap = 0;
adapter_height=9;

draw_holder = 1;
draw_ball_joint = 1;
draw_lamp_adapter = 1;
draw_knob = 1;
draw_camera_knob = 1;
draw_cap = 1;


if(draw_lamp_adapter) translate([0,0,-44]) rotate([0,0,90]) lamp_adapter();


if(draw_ball_joint) ball_joint();
if(draw_cap) ball_cap();

if(draw_knob) translate([-(ball_radius+15),0,0]) rotate([0,-90,0]) knob();

if(draw_holder) holder();

if(draw_camera_knob) translate([0,0,ball_radius+9]) camera_knob();

module camera_knob()
{
    $fn=50;
    
    difference()
    {
        union()
        {
            difference()
            {
                cylinder(r=18, h=4.75+1.5, $fn=30);
                for(i=[0:5])
                    rotate([0,0,(360/6)*i]) translate([21,0,-.1]) cylinder(r=5,h=10.2, $fn=30);
                translate([0,0,-0.1]) cylinder_outer(height=4.75, radius=(11+tolerance)/2, fn=6);
                
            }
            //translate([0,0,-6]) cylinder(r=12,h=6,$fn=30);
        }
    translate([0,0,-8]) screw_shaft();
    }
}


module holder()
{
    difference()
{
    union()
    {
        translate([ball_radius-holder_inset,0,0]) rotate([90,180,90])
        {
                cylinder(r=ball_radius*(2/3), h=6, $fn=50); //holder arm
                translate([0,(ball_radius+holder_length)/2,3]) cube(size = [ball_radius*2*(2/3),ball_radius+holder_length,6],center=true);
        }
            
        translate([-(ball_radius-holder_inset),0,0]) rotate([-90,0,90]) 
        {
                cylinder(r    =ball_radius*(2/3), h=6, $fn=50); //holder arm
                translate([0,(ball_radius+holder_length)/2,3]) cube(size = [ball_radius*2*(2/3),ball_radius+holder_length,6],center=true);
        }
        //holder base (0.1 added to the height to create overlap with the arms, renders better
        hull()
        {
            translate([0,0,-(ball_radius+holder_length)-holder_height/2]) cube(size=[24.25,ball_radius*2,holder_height+.1], center=true);
            translate([ball_radius+2,0,-(ball_radius+holder_length)-holder_height/2]) cube(size=[6,ball_radius*2*(2/3),holder_height+.1], center=true);
            translate([-(ball_radius+2),0,-(ball_radius+holder_length)-holder_height/2]) cube(size=[6,ball_radius*2*(2/3),holder_height+.1], center=true);
        }
    }
    sphere(r=ball_radius+tolerance+holder_gap, $fn=50);
    rotate([0,90,0]) cylinder(r=(6.15 + tolerance)/2, h=100, center=true, $fn=26);
    
    
    translate([0,0,-(ball_radius+holder_length+holder_height)]) rotate([0,0,90])
    {
            translate([24.25/2,0,-.5]) cylinder(r=2.125,h=adapter_height+1,$fn=20);
            translate([-24.25/2,0,-.5]) cylinder(r=2.125,h=adapter_height+1,$fn=20);
            
            translate([24.25/2,0,holder_height-4]) cylinder(d=7.5,h=4.1,$fn=20);
            translate([-24.25/2,0,holder_height-4]) cylinder(d=7.5,h=4.1,$fn=20);
        
            
    }
    
    translate([ball_radius+6-1-3,0,0]) rotate([90,90,90]) cylinder_outer(height=5, radius=(11+tolerance)/2, fn=6);
    
    
}
}


module knob()
{
    $fn=50;
    
    difference()
    {
        union()
        {
            difference()
            {
                cylinder(r=18, h=10, $fn=30);
                for(i=[0:5])
                    rotate([0,0,(360/6)*i]) translate([21,0,-.1]) cylinder(r=5,h=10.2, $fn=30);
                translate([0,0,-6+3+.1]) cylinder_outer(height=15, radius=(11+tolerance)/2, fn=6);
                
            }
            translate([0,0,-6]) cylinder(r=12,h=6,$fn=30);
        }
    translate([0,0,-8]) screw_shaft();
    }
}

module ball_cap()
{
    translate([0,0,ball_radius-1])
    {
        difference()
        {
            cylinder(r=8+2+tolerance,h=6+2, $fn=20);
            cylinder(r=8+tolerance,h=6, $fn=20);
            cylinder(r=(6.15 + tolerance)/2, h=20, $fn=20);
        }
    }
}

module ball_joint()
{



difference()
{
    union()
    {
        sphere(r=ball_radius, $fn=50); //main ball
        translate([0,0,ball_radius-5]) cylinder(r=8,h=10,$fn=20);
    }
    rotate([0,90,0]) cylinder(r=(6.15 + tolerance)/2, h=35, center=true, $fn=26);
    
    hull(){
        translate([0,0,5]) screw_head(); //slot for the camera 1/4 bolt
        translate([0,20,5]) screw_head();
    }
    hull(){
        translate([0,0,5]) screw_shaft(); //slot fo the camera 1/4 bolt
        translate([0,20,5]) screw_shaft();
    }
    
    rotate([0,90,0]) cylinder(r=(6.15 + tolerance)/2, h=100, center=true, $fn=26); //1-4 hole through ball
}


}

module screw_head()
{
    rotate([0,0,360/12]) cylinder_outer(height=4.75, radius=(11+tolerance)/2, fn=6);
}

module screw_shaft()
{
     cylinder(r=(6.15 + tolerance)/2, h=25, $fn=20);
}

   
module lamp_adapter()
{
    

    
    difference()
    {
        hull()
        {
            translate([24.25/2,0,0]) cylinder(r=8.16,h=adapter_height,$fn=20);
            translate([-24.25/2,0,0]) cylinder(r=8.16,h=adapter_height,$fn=20);
        }
            translate([24.25/2,0,-.5]) cylinder(r=2.125,h=adapter_height+1,$fn=20);
            translate([-24.25/2,0,-.5]) cylinder(r=2.125,h=adapter_height+1,$fn=20);
            
            translate([24.25/2,0,-.1]) cylinder(d=7.5,h=4.1,$fn=20);
            translate([-24.25/2,0,-.1]) cylinder(d=7.5,h=4.1,$fn=20);
        
            translate([0,0,adapter_height - 4.74]) screw_head();
            translate([0,0,-0.01]) screw_shaft();
    }
        
}

module cylinder_outer(height,radius,fn){
   fudge = 1/cos(180/fn);
   cylinder(h=height,r=radius*fudge,$fn=fn);}